/* GNU polyxmass - the massist's program.
   -------------------------------------- 
   Copyright (C) 2000,2001,2002,2003,2004 Filippo Rusconi

   http://www.polyxmass.org

   This file is part of the "GNU polyxmass" project.
   
   The "GNU polyxmass" project is an official GNU project package (see
   www.gnu.org) released ---in its entirety--- under the GNU General
   Public License and was started at the Centre National de la
   Recherche Scientifique (FRANCE), that granted me the formal
   authorization to publish it under this Free Software License.

   This software is free software; you can redistribute it and/or
   modify it under the terms of the GNU  General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU  General Public
   License along with this software; if not, write to the
   Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/
#include "polyxedit-ui-pka-ph-pi.h"
#include "polyxmass-ui-seqed-widget.h"
#include "polyxedit-ui-seqed-wnd.h"
#include "polyxedit-acidobasic.h"



GtkWidget *
polyxedit_pka_ph_pi_wnd_setup (PxmEditCtxt *editctxt)
{
  GtkWidget *widget = NULL;
  GtkWidget *window = NULL;
  GtkWidget *seqed_widget = NULL;

  GladeXML *xml = NULL;

  gchar *gui_file = NULL;
  gchar *help = NULL;

  gdouble *positive_charges = NULL;
  gdouble *negative_charges = NULL;
  


  g_assert (editctxt != NULL);

  seqed_widget = 
    g_object_get_data (G_OBJECT (editctxt->sequence_editor_wnd), "seqed_widget");
  g_assert (seqed_widget != NULL);
  



  gui_file = 
    g_strdup_printf ("%s/polyxedit-acidobasic.glade", userspec->gladedir);

  xml = glade_xml_new (gui_file, "ph_pka_net_charge_pi_wnd", 
		       PACKAGE);

  g_free (gui_file);
  
  if (xml == NULL)
    {
      g_error (_("%s@%d: failed to load the interface\n"),
	     __FILE__, __LINE__);

      g_free (gui_file);
      
      return NULL;
    }
  
  window = glade_xml_get_widget (xml, "ph_pka_net_charge_pi_wnd");
  
  if (window == NULL)
    {
      g_critical (_("%s@%d: failed to create the acido-basicity "
		    "calculations configuration window\n"),
	     __FILE__, __LINE__);
      
      g_object_unref (G_OBJECT (xml));
      
      return NULL;
    }

  /* Immediately set to the window a pointer to the editctxt:
   */
  g_object_set_data (G_OBJECT (window), "editctxt", editctxt);

  /* Immediately allocate the doubles that we will feed continuously 
     when performing calculations...
  */
  positive_charges = g_malloc0 (sizeof (gdouble));
  g_object_set_data_full (G_OBJECT (window),
			  "positive_charges", positive_charges, g_free);
  
  negative_charges = g_malloc0 (sizeof (gdouble));
  g_object_set_data_full (G_OBJECT (window),
			  "negative_charges", negative_charges, g_free);
  
  
  /* We may need to displayed timed-out messages in here:
   */
  widget = glade_xml_get_widget (xml, "messages_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "messages_entry", widget);
  
  

  /* The vertical box where the polymer sequence data widgets are packed, 
     so that we make them invisible in one go.
  */
  widget = glade_xml_get_widget (xml, "polseq_prop_vbox");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window), 
		     "polseq_prop_vbox", widget);
  

  /* Set the name of the polymer sequence to its correspondent GtkEntry.
   */
  widget = glade_xml_get_widget (xml, "polseq_name_entry"); 
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window), 
		     "polseq_name_entry", widget);

  g_assert (editctxt->polymer->plminfo->name != NULL);
  
  gtk_entry_set_text (GTK_ENTRY (widget), 
		      editctxt->polymer->plminfo->name);

  /* Set the code of the polymer sequence to its correspondent GtkEntry.
   */
  widget = glade_xml_get_widget (xml, "polseq_code_entry"); 
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window), 
		     "polseq_code_entry", widget);

  g_assert (editctxt->polymer->plminfo->code != NULL);
  
  gtk_entry_set_text (GTK_ENTRY (widget), 
		      editctxt->polymer->plminfo->code);


  /* Set the identity number (actually the seqed_widget pointer) to 
   * its GtkEntry (this is useful when cleavages are made to identify
   * unambiguously the polymer from which results are displayed).
   */
  widget = glade_xml_get_widget (xml, "polseq_id_number_entry");
  g_object_set_data (G_OBJECT (window), 
		     "polseq_id_number_entry",
		     widget);
  help = g_strdup_printf ("%p", seqed_widget);
  gtk_entry_set_text (GTK_ENTRY (widget), help);
  g_free (help);


  
  widget = glade_xml_get_widget (xml, "update_sequence_region_button");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "update_sequence_region_button", widget);

  g_signal_connect (G_OBJECT (widget),
		    "clicked",
		    G_CALLBACK 
		    (polyxedit_pka_ph_pi_wnd_sequence_region_button), 
		    window);

  widget = glade_xml_get_widget (xml, "sel_start_pos_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "sel_start_pos_entry", widget);
  
  widget = glade_xml_get_widget (xml, "sel_end_pos_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "sel_end_pos_entry", widget);
  
  widget = glade_xml_get_widget (xml, "whole_sequence_checkbutton");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "whole_sequence_checkbutton", widget);

  /* At this point we should be able to set the values to the text
     entries above, if some portion of the protein is already selected...
  */
  polyxedit_pka_ph_pi_wnd_sequence_region_button (NULL,
						  window);
  

  widget = glade_xml_get_widget (xml, "hydrogen_potential_spinbutton");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "hydrogen_potential_spinbutton", widget);

  widget = glade_xml_get_widget (xml, "isoelectric_point_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "isoelectric_point_entry", widget);

  widget = glade_xml_get_widget (xml, "treated_pka_number_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "treated_pka_number_entry", widget);

  widget = glade_xml_get_widget (xml, "positive_charges_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "positive_charges_entry", widget);

  widget = glade_xml_get_widget (xml, "negative_charges_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "negative_charges_entry", widget);

  widget = glade_xml_get_widget (xml, "net_charge_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "net_charge_entry", widget);

  widget = glade_xml_get_widget (xml, "calculate_net_charge_button");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "calculate_net_charge_button", widget);

  g_signal_connect (G_OBJECT (widget),
		    "clicked",
		    G_CALLBACK 
		    (polyxedit_pka_ph_pi_wnd_calculate_net_charge_button), 
		    window);

  widget = glade_xml_get_widget (xml, "calculate_isoelectric_point_button");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window),
		     "calculate_isoelectric_point_button", widget);

  g_signal_connect (G_OBJECT (widget),
		    "clicked",
		    G_CALLBACK 
		    (polyxedit_pka_ph_pi_wnd_calculate_pi_button), 
		    window);


  gtk_widget_show_all (GTK_WIDGET (window));


  /* Signal / callback connections.
   */
  g_signal_connect (G_OBJECT (window),
		    "delete_event",
		    G_CALLBACK (polyxedit_pka_ph_pi_wnd_delete_event), 
		    editctxt);
  
  /* Signal / callback connections.
   */
  g_signal_connect (G_OBJECT (window),
		    "destroy_event",
		    G_CALLBACK (polyxedit_pka_ph_pi_wnd_destroy_event), 
		    editctxt);
  


  /* We have finished setting up the window, and so also using the xml
     data, unref them.
   */
  g_object_unref (G_OBJECT (xml));
  
  /* Set this window pointer as a full datum to the polymer sequence
     editor window, so that when it is closed this window is closed
     also. 

     There might be more than one monomer modification window opened
     for a given polymer seqence editing window, and we do not want
     that the second window destroys the datum name of the first
     window, so we create an uambiguous datum name each time.
  */
  help = g_strdup_printf ("pka_ph_pi_wnd-%p", window);
  
  g_object_set_data_full 
    (G_OBJECT (editctxt->sequence_editor_wnd),
     help, GTK_WIDGET (window), 
     (GDestroyNotify) polyxedit_pka_ph_pi_wnd_really_close);
  
  g_free (help);
  
  return window;
}
  

gboolean
polyxedit_pka_ph_pi_wnd_get_sequence_interval (GtkWidget *window,
						gint *start_idx,
						gint *end_idx)
{
  GtkWidget *widget = NULL;

  gchar *help = NULL;
  gint len = 0;

  gint start = 0;
  gint end = 0;
  
  PxmEditCtxt *editctxt = NULL;

  


  g_assert (window != NULL);
  
  editctxt = g_object_get_data (G_OBJECT (window), "editctxt");
  g_assert (editctxt != NULL);

  g_assert (start_idx != NULL && end_idx != NULL);
  

  g_assert (editctxt->polymer != NULL);
  g_assert (editctxt->polymer->monomerGPA != NULL);
  len = editctxt->polymer->monomerGPA->len;

  if (len == 0)
    {
      return -1;
    }
  

  /* We must check if the user wants that the calculation be made for
     the whole sequence or for the selection region of the polymer sequence
     only, or for the interval that is defined with the start_pos and 
     end_pos entry widgets.
  */
  widget = 
    g_object_get_data (G_OBJECT (window), "whole_sequence_checkbutton");
  g_assert (widget != NULL);

  if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    {
      start = 0;
      end = len;
    }
  else
    {
      /* We should check what the start_pos and end_pos entries say:
       */
      widget = g_object_get_data (G_OBJECT (window), "sel_start_pos_entry");
      g_assert (widget != NULL);
      
      help = gtk_editable_get_chars (GTK_EDITABLE (widget), 0, -1);
      g_assert (help != NULL);
      
      if (strlen (help) <= 0)
	{
	  start = 0;
	}
      else
	{
	  /* There is a string in the entry. Convert it to gint.
	   */
	  if (FALSE == libpolyxmass_globals_strtoi (help, &start, 10))
	    start = 0;
	  else
	    {
	      /* Remember that the user "counts" monomers starting at 1,
		 and we need to count starting at 0, because we use indexes
		 and not positions. So decrement by one the value entered
		 by the user.
	      */
	      start--;
	    }
	}
      
      widget = g_object_get_data (G_OBJECT (window), "sel_end_pos_entry");
      g_assert (widget != NULL);
      
      help = gtk_editable_get_chars (GTK_EDITABLE (widget), 0, -1);
      g_assert (help != NULL);
      
      if (strlen (help) <= 0)
	{
	  end = 0;
	}
      else
	{
	  /* There is a string in the entry. Convert it to gint.
	   */
	  if (FALSE == libpolyxmass_globals_strtoi (help, &end, 10))
	    end = 0;
	  else
	    {
	      /* Remember that for the *end_idx, the calculations need
		 to have it incremented by one with respect to the real
		 last monomer index. So, contrary to start_idx, we do not
		 change its value.
		 
		 end;
	      */
	    }
	}

      /* Now make some final checks:
       */

      if (start > len)
	start = len;

      if (start < 0)
	start = 0;
      
      if (end > len)
	end = len;

      if (end < 0)
	end = 0;
    }

  *start_idx = start;
  *end_idx = end;
  
  return TRUE;
}


void
polyxedit_pka_ph_pi_wnd_sequence_region_button (GtkWidget *button,
						gpointer data)
{
  GtkWidget *window = data;
  GtkWidget *widget = NULL;
  
  PxmEditCtxt *editctxt = NULL;

  PxmSeqedWidget *seqed_widget = NULL;
  
  gchar *help = NULL;
  
  gint start = 0;
  gint end = 0;
  
  g_assert (window != NULL);
  
  editctxt = g_object_get_data (G_OBJECT (window), "editctxt");
  g_assert (editctxt != NULL);
  
  seqed_widget = g_object_get_data (G_OBJECT (editctxt->sequence_editor_wnd), 
				    "seqed_widget");
  g_assert (seqed_widget != NULL);


  /* Prepare the variables from the sequence editor window, so that
     we can put them immediately in the start/end positions's entry
     widgets.
  */
  if (FALSE != polyxmass_seqed_widget_get_selection_indices (GTK_WIDGET (seqed_widget),
							     &start, &end))
    {
      /* Remember that due to the handling of the polymer sequence
	 displayed in the sequence editor, the end value that is 
	 returned is indeed corresponding to the real index of the last
	 selected monomer + 1, so care should be taken not to outbound
	 the polymer sequence array.
      */
      
      /* We show the (start + 1) value because we want to show
	 a position and not an index, while the end needs not be
	 incremented because it is by itself already incremented.
      */      
      widget = g_object_get_data (G_OBJECT (window), "sel_start_pos_entry");
      g_assert (widget != NULL);

      help = g_strdup_printf ("%d", start + 1);
      gtk_entry_set_text (GTK_ENTRY (widget), help);
      g_free (help);

      widget =  g_object_get_data (G_OBJECT (window), "sel_end_pos_entry");
      g_assert (widget != NULL);
      help = g_strdup_printf ("%d", end);
      gtk_entry_set_text (GTK_ENTRY (widget), help);
      g_free (help);

      /* If there is a sequence selected, by default we disable the
	 "whole sequence" checkbutton. Otherwise we enable it.
      */
      widget = g_object_get_data (G_OBJECT (window), 
				  "whole_sequence_checkbutton");
      g_assert (widget != NULL);

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
    }
  else
    {
      widget = g_object_get_data (G_OBJECT (window), "sel_start_pos_entry");
      g_assert (widget != NULL);

      widget = g_object_get_data (G_OBJECT (window), "sel_end_pos_entry");
      g_assert (widget != NULL);

      /* If there is a sequence selected, by default we disable the
	 "whole sequence" checkbutton. Otherwise we enable it.
      */
      widget = g_object_get_data (G_OBJECT (window), 
				  "whole_sequence_checkbutton");
      g_assert (widget != NULL);
      
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
    }
  
  return;
}


/* Returns -1 if an error occurs. Returns the number of pka that were
   processed.
 */
gint
polyxedit_pka_ph_pi_wnd_calculate_net_charge_button (GtkWidget *button,
						   gpointer data)
{
  GtkWidget *window = data;
  GtkWidget *pi_widget = NULL;
  GtkWidget *ph_widget = NULL;
  GtkWidget *charges_widget = NULL;

  PxmPolchemdefCtxt *polchemdefctxt = NULL;
  PxmPolymer *polymer = NULL;
  PxmEditCtxt *editctxt = NULL;
  PxmCalcOpt *calcopt = NULL;

  gint processed_chemgroups = 0;

  gchar *help = NULL;
  
  gdouble ph = 0;
  gdouble *positive_charges = NULL;
  gdouble *negative_charges = NULL;
    
  /* We are asked to calculate the net charge of the polymer sequence
     (or its interval) for the pH currently displayed in the
     corresponding spinbutton widget.
  */
  g_assert (window != NULL);
  
  /* Note that the charges holders (gdouble *) are allocated when the
     window is set up, so we first have to get the pointers out of the
     window ! Of course we have to reset them to 0 prior to performing
     the new calculation.
  */
  negative_charges = g_object_get_data (G_OBJECT (window),
					"negative_charges");
  g_assert (negative_charges != NULL);
  *negative_charges = 0;
  positive_charges = g_object_get_data (G_OBJECT (window),
					"positive_charges");
  g_assert (positive_charges != NULL);
  *positive_charges = 0;


  editctxt = g_object_get_data (G_OBJECT (window), "editctxt");
  g_assert (editctxt != NULL);
  polymer = editctxt->polymer;
  g_assert (polymer != NULL);
  polchemdefctxt = editctxt->polchemdefctxt;
  g_assert (polchemdefctxt != NULL);

  


  ph_widget =  g_object_get_data (G_OBJECT (window), 
				  "hydrogen_potential_spinbutton");
  g_assert (ph_widget != NULL);
  
  ph = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ph_widget));


  if (ph <= 0 || ph >= 14)
    {
      polyxmass_timeoutmsg_message_set ((GtkWindow *) window,
					_("pH value is not acceptable."),
					POLYXMASS_MEDI_MSG_TIMEOUT);

      /* We do not want that the user makes fake reports if no
	 calculation was succcessful. See the end of the function for
	 an explanation of the function call below. It will
	 automagically free any old calcopt instance previously stored
	 as datum in the 'window'. Thus, when the user will try to
	 make a report, it will not be made because no 'calcopt'
	 pointer is available in the 'window'.
      */
      g_object_set_data (G_OBJECT (window), "calcopt", NULL);
      
      return -1;
    }
  

  /* Duplicate the editctxt's calcopt (because its start_idx, end_idx
     members are going to be modified during the cleavage operation):
   */
  calcopt = pxmchem_calcopt_dup (editctxt->calcopt);

  /* Get the sequence interval for which the calculation should be
     performed.
   */
  if (FALSE ==  
      polyxedit_pka_ph_pi_wnd_get_sequence_interval (window,
						     &calcopt->start_idx,
						     &calcopt->end_idx))
  {
    polyxmass_timeoutmsg_message_set ((GtkWindow *) window,
				      _("Failed to get sequence interval."),
				      POLYXMASS_MEDI_MSG_TIMEOUT);
    
    /* Free this newly allocated calcopt object.
     */
    pxmchem_calcopt_free (calcopt);
    
    /* We do not want that the user makes fake reports if no
       calculation was succcessful. See the end of the function
       for an explanation of the function call below. It will
       automagically free any old calcopt instance previously
       stored as datum in the 'window'. Thus, when the user will
       try to make a report, it will not be made because no
       'calcopt' pointer is available in the 'window'.
    */
    g_object_set_data (G_OBJECT (window), "calcopt", NULL);
    
    return -1;
  }
  
  /* At this point we have all the requisites to work on the net charge
     calculation.
  */
  processed_chemgroups =
    polyxedit_acidobasic_polseq_calculate_net_charge (editctxt, ph,
						    calcopt,
						    negative_charges,
						    positive_charges);
  if (-1 == processed_chemgroups)
    {
      polyxmass_timeoutmsg_message_set ((GtkWindow *) window,
					_("Failed to calculate net charge."),
					POLYXMASS_MEDI_MSG_TIMEOUT);
      
      /* Free this newly allocated calcopt object.
       */
      pxmchem_calcopt_free (calcopt);
      
      /* We do not want that the user makes fake reports if no
	 calculation was succcessful. See the end of the function
	 for an explanation of the function call below. It will
	 automagically free any old calcopt instance previously
	 stored as datum in the 'window'. Thus, when the user will
	 try to make a report, it will not be made because no
	 'calcopt' pointer is available in the 'window'.
      */
      g_object_set_data (G_OBJECT (window), "calcopt", NULL);
      
      return -1;
    }

  /*
  debug_printf (("The charges are: "
		 "negative = %lf ; positive = %lf ; net = %lf\n"
		 "Number of processed pkas = %d\n",
		 *negative_charges, *positive_charges, 
		 *positive_charges + *negative_charges,
		 processed_chemgroups));
  */
  
  /* Immediately make clear that we have calculated the pI by setting to
     nothing the text in the pH combo-entry.
  */
  pi_widget = g_object_get_data (G_OBJECT (window), 
			      "isoelectric_point_entry");
  g_assert (pi_widget != NULL);

  gtk_entry_set_text (GTK_ENTRY (pi_widget), "0.00000");
  

  charges_widget = g_object_get_data (G_OBJECT (window), 
				      "negative_charges_entry");
  g_assert (charges_widget != NULL);
  
  help = g_strdup_printf ("%lf", *negative_charges);
  
  gtk_entry_set_text (GTK_ENTRY (charges_widget), help);
  
  g_free (help);

  charges_widget = g_object_get_data (G_OBJECT (window), 
				      "positive_charges_entry");
  g_assert (charges_widget != NULL);
  
  help = g_strdup_printf ("%lf", *positive_charges);
  
  gtk_entry_set_text (GTK_ENTRY (charges_widget), help);
  
  g_free (help);

  charges_widget = g_object_get_data (G_OBJECT (window), 
				      "net_charge_entry");
  g_assert (charges_widget != NULL);
  
  help = g_strdup_printf ("%lf", *positive_charges + *negative_charges);
  
  gtk_entry_set_text (GTK_ENTRY (charges_widget), help);
  
  g_free (help);
  
  g_object_set_data (G_OBJECT (window), "calcopt", calcopt);
      
  polyxmass_timeoutmsg_message_set ((GtkWindow *) window,
				    _("Operation completed."),
				    POLYXMASS_NORM_MSG_TIMEOUT);

  /* Since we might have to write a report about this calculation, we
     have to be able to know with which calculation  options
     the calculation was performed. Thus, we store this information as
     full data to the window, for later enquiry. See
     polyxedit_pka_ph_pi_wnd_make_report_text_format() for details !

     Note that the g_object_set_data_full call below will elicit
     freeing of potentially pre-existing instances using their
     respective freeing functions.
  */
  g_object_set_data_full (G_OBJECT (window), "calcopt", calcopt,
			  (GDestroyNotify) pxmchem_calcopt_free);

  return processed_chemgroups;
}




gint
polyxedit_pka_ph_pi_wnd_calculate_pi_button (GtkWidget *button,
					   gpointer data)
{
  GtkWidget *window = data;
  GtkWidget *messages_entry = NULL;

  PxmEditCtxt *editctxt = NULL;
  PxmCalcOpt *calcopt = NULL;

  gint processed_chemgroups = 0;
  gint iteration = 0;

  gdouble tested_ph = 0;
  
  gdouble charge_first = 0;
  gdouble charge_third = 0;

  gdouble *positive_charges = NULL;
  gdouble *negative_charges = NULL;

  GtkWidget *widget = NULL;

  gchar *help = NULL;

  

  

  /* We are asked to calculate the isoelectric point for the current
     polymer sequence (or a region of it). 

     The way we do this is start by setting pH to 7.0, calculate the net
     charge of the sequence at this pH and then set the pH to
     higher/lower value if the charge is farther away (positive or
     negative) than 0.05.
  */

  g_assert (window != NULL);


  /* Note that the charges holders (gdouble *) are allocated when the
     window is set up, so we first have to get the pointers out of the
     window ! Of course we have to reset them to 0 prior to performing
     the new calculation.
  */
  negative_charges = g_object_get_data (G_OBJECT (window),
					"negative_charges");
  g_assert (negative_charges != NULL);
  *negative_charges = 0;
  
  positive_charges = g_object_get_data (G_OBJECT (window),
					"positive_charges");
  g_assert (positive_charges != NULL);
  *positive_charges = 0;
  
  editctxt = g_object_get_data (G_OBJECT (window), "editctxt");
  g_assert (editctxt != NULL);

  /* Duplicate the editctxt's calcopt (because its start_idx, end_idx
     members are going to be modified during the cleavage operation):
   */
  calcopt = pxmchem_calcopt_dup (editctxt->calcopt);

  /* Get the sequence interval for which the calculation should be
     performed.
   */
  if (FALSE ==  
      polyxedit_pka_ph_pi_wnd_get_sequence_interval (window,
						     &calcopt->start_idx,
						     &calcopt->end_idx))
  {
    polyxmass_timeoutmsg_message_set ((GtkWindow *) window,
				      _("Failed to get sequence interval."),
				      POLYXMASS_MEDI_MSG_TIMEOUT);
    
    /* Free this newly allocated calcopt object.
     */
    pxmchem_calcopt_free (calcopt);
    
    /* We do not want that the user makes fake reports if no
       calculation was succcessful. See the end of the function
       for an explanation of the function call below. It will
       automagically free any old calcopt instance previously
       stored as datum in the 'window'. Thus, when the user will
       try to make a report, it will not be made because no
       'calcopt' pointer is available in the 'window'.
    */
    g_object_set_data (G_OBJECT (window), "calcopt", NULL);
    
    return -1;
  }
  
  messages_entry = g_object_get_data (G_OBJECT (window), "messages_entry");
  g_assert (messages_entry != NULL);
    

  while (1)
    {

#if 0
      help = g_strdup_printf (_("Current pH being tested: %lf"), tested_ph);
      gtk_entry_set_text (GTK_ENTRY (messages_entry), help);
      debug_printf (("%s\n", help));
      g_free (help);
#endif

      processed_chemgroups =
	polyxedit_acidobasic_polseq_calculate_net_charge (editctxt, tested_ph,
							calcopt,
							negative_charges,
							positive_charges);
      if (-1 == processed_chemgroups)
	{
	  polyxmass_timeoutmsg_message_set ((GtkWindow *) window,
					    _("Failed to calculate net charge."),
					    POLYXMASS_MEDI_MSG_TIMEOUT);
	  
	  /* Free this newly allocated calcopt object.
	   */
	  pxmchem_calcopt_free (calcopt);
	  
	  /* We do not want that the user makes fake reports if no
	     calculation was succcessful. See the end of the function
	     for an explanation of the function call below. It will
	     automagically free any old calcopt instance previously
	     stored as datum in the 'window'. Thus, when the user will
	     try to make a report, it will not be made because no
	     'calcopt' pointer is available in the 'window'.
	  */
	  g_object_set_data (G_OBJECT (window), "calcopt", NULL);
	  
	  return -1;
	}
      
      /* Note that if the 0.001 tested_ph step is enough to switch
	 the net charge from one excess value to another excess value
	 in the opposite direction, we'll enter an infinite loop.

	 The evidence for such loop is that every other two measures,
	 the net charge of the polymer sequence will be the same.

	 Here we test this so that we can break the loop.
      */
      iteration++;
      
      if (iteration == 1)
	{
	  charge_first = (*positive_charges + *negative_charges);
	}
      else if (iteration == 3)
	{
	  charge_third = (*positive_charges + *negative_charges);
	  
	  if (charge_third == charge_first)
	    {
	      /*
	      debug_printf (("Exiting loop with special case\n"));
	      */
	      break;
	    }
	    
	  iteration = 0;
	  
	  charge_first = (*positive_charges + *negative_charges);
	}
      
            
      /* At this point we have to test the net charge:
       */
      if ((*positive_charges + *negative_charges) >= -0.1
	  && (*positive_charges + *negative_charges) <= 0.1)
	break;
      
      if ((*positive_charges + *negative_charges) > 0)
	{
	  /* The tested_ph is too acidic.
	   */
	  tested_ph += 0.01;
	  
	  *positive_charges = 0;
	  *negative_charges = 0;
	  
	  continue;
	}
      
      if ((*positive_charges + *negative_charges) < 0)
	{
	  /* The tested_ph is too basic.
	   */
	  tested_ph -= 0.01;
	  
	  *positive_charges = 0;
	  *negative_charges = 0;

	  continue;
	}
    }
  
  /* At this point, we should have calculated the pI.
   */
  /* Immediately make clear that we have calculated the pI by setting to
     nothing the text in the pH combo-entry.
  */
  widget = g_object_get_data (G_OBJECT (window), 
			      "hydrogen_potential_spinbutton");
  g_assert (widget != NULL);
  
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), 0.0);


  widget = g_object_get_data (G_OBJECT (window), 
				      "negative_charges_entry");
  g_assert (widget != NULL);
  
  help = g_strdup_printf ("%lf", *negative_charges);
  
  gtk_entry_set_text (GTK_ENTRY (widget), help);
  
  g_free (help);


  widget = g_object_get_data (G_OBJECT (window), 
				      "positive_charges_entry");
  g_assert (widget != NULL);
  
  help = g_strdup_printf ("%lf", *positive_charges);
  
  gtk_entry_set_text (GTK_ENTRY (widget), help);
  
  g_free (help);


  widget = g_object_get_data (G_OBJECT (window), 
				      "net_charge_entry");
  g_assert (widget != NULL);
  
  help = g_strdup_printf ("%lf", *positive_charges + *negative_charges);
  
  gtk_entry_set_text (GTK_ENTRY (widget), help);
  
  g_free (help);


  widget = g_object_get_data (G_OBJECT (window), 
			      "isoelectric_point_entry");
  g_assert (widget != NULL);
  
  help = g_strdup_printf ("%lf", tested_ph);
  
  gtk_entry_set_text (GTK_ENTRY (widget), help);

  g_free (help);
  

  widget = g_object_get_data (G_OBJECT (window), 
			      "treated_pka_number_entry");
  g_assert (widget != NULL);
  
  help = g_strdup_printf ("%d", processed_chemgroups);
  
  gtk_entry_set_text (GTK_ENTRY (widget), help);
  
  g_free (help);
  
  
  polyxmass_timeoutmsg_message_set ((GtkWindow *) window,
				    _("Calculation finished."),
				    POLYXMASS_MEDI_MSG_TIMEOUT);
  

  /* Since we might have to write a report about this calculation, we
     have to be able to know with which calculation  options
     the calculation was performed. Thus, we store this information as
     full data to the window, for later enquiry. See
     polyxedit_pka_ph_pi_wnd_make_report_text_format() for details !

     Note that the g_object_set_data_full call below will elicit
     freeing of potentially pre-existing instances using their
     respective freeing functions.
  */
  g_object_set_data_full (G_OBJECT (window), "calcopt", calcopt,
			  (GDestroyNotify) pxmchem_calcopt_free);
  
  return processed_chemgroups;
}


/* REPORTING FUNCTIONS.
 */
gchar *
polyxedit_pka_ph_pi_wnd_make_report (PxmReportOpt* reportopt,
				     PxmWinMngmt* winmngmt)
{
  


  g_assert (winmngmt != NULL);
  g_assert (reportopt != NULL);
  
  g_assert (winmngmt->seqed_widget != NULL);
  g_assert (winmngmt->wnd != NULL);

  /* Great, we are able to start doing the report. We need to know what
     kind of report we want : text or LaTeX.
  */
  if (reportopt->export_format == EXPORT_FORMAT_TEXT)
    return polyxedit_pka_ph_pi_wnd_make_report_text_format (reportopt,
							    winmngmt);
  else if (reportopt->export_format == EXPORT_FORMAT_LATEX)
    return polyxedit_pka_ph_pi_wnd_make_report_latex_format (reportopt,
							     winmngmt);
  else
    g_assert_not_reached ();

  return NULL;
}


gchar *
polyxedit_pka_ph_pi_wnd_make_report_text_format (PxmReportOpt* reportopt,
						 PxmWinMngmt* winmngmt)
{
  GtkWidget *window = NULL;
  GtkWidget *widget = NULL;
  GtkWidget *seqed_widget = NULL;

  gchar *report = NULL;
  
  gdouble ph = 0;
  
  GString *report_gs = NULL;

  PxmPolymer *polymer = NULL;
  PxmPlminfo *plminfo = NULL;
  PxmCalcOpt *calcopt = NULL;



  g_assert (winmngmt != NULL);
  g_assert (reportopt != NULL);
  
  seqed_widget = winmngmt->seqed_widget;
  g_assert (seqed_widget != NULL);
  
  window = winmngmt->wnd ;
  g_assert (window != NULL);
  
  polymer = PXM_SEQED_WIDGET (seqed_widget)->polymer;
  g_assert (polymer != NULL);

  plminfo = polymer->plminfo;
  g_assert (plminfo != NULL);

  
  report_gs = g_string_new ("");
  
  report_gs = pxmchem_polymer_format_txt_string_polseqinfo (plminfo,
							    reportopt,
							    report_gs);
  
  /* Get the pointer to the structure we know have determined how the
     calculation was performed. However, the pointer is non-NULL only
     if at least one calculation was performed, since it is set as
     full data when the calculation is actually performed.

     Thus, if this is NULL, we return with a message...
  */
  calcopt = g_object_get_data (G_OBJECT (window), "calcopt");
  
  if  (calcopt == NULL)
    {
      g_warning (_("%s@%d: no calculation took place, nothing to report\n"),
		 __FILE__, __LINE__);
      
      report_gs = 
	g_string_append (report_gs, _("No calculation took place, "
			 "nothing to report\n"));
      
      report = report_gs->str;
      g_string_free (report_gs, FALSE);
      
      return report;
    }

  if ((reportopt->polymer_opt & PXM_PLM_REPORTOPT_CALCOPTIONS) ==
      PXM_PLM_REPORTOPT_CALCOPTIONS)
    {
      report_gs = 
	g_string_append (report_gs, _("Calculations Options:\n"
			 "---------------------\n"));

      if (calcopt->capping & PXM_CAPPING_RIGHT && 
	  calcopt->capping & PXM_CAPPING_LEFT)
	report_gs = 
	  g_string_append (report_gs, _("Capping: BOTH\n"));
      
      else if (calcopt->capping & PXM_CAPPING_RIGHT)
	report_gs = 
	  g_string_append (report_gs, _("Capping: RIGHT\n"));
      
      else if (calcopt->capping & PXM_CAPPING_LEFT)
	report_gs = 
	  g_string_append (report_gs, _("Capping: LEFT\n"));
      
      else
	report_gs = g_string_append (report_gs, _("Capping: NONE\n"));
	
      if (calcopt->mnm_chement & PXMCHEMENT_MNM_MODIF)
	report_gs = 
	  g_string_append (report_gs, _("Monomer Chemical Entities: MODIFS\n"));
      else
	report_gs = 
	  g_string_append (report_gs, _("Monomer Chemical Entities: NONE\n"));
	
	
      if (calcopt->plm_chement & PXMCHEMENT_PLM_LEFT_MODIF
	  && calcopt->plm_chement & PXMCHEMENT_PLM_RIGHT_MODIF)
	report_gs = 
	  g_string_append (report_gs, _("Polymer End Modifs: BOTH\n"));

      else if (calcopt->plm_chement & PXMCHEMENT_PLM_LEFT_MODIF)
	report_gs = 
	  g_string_append (report_gs, _("Polymer End Modif: LEFT\n"));

      else if (calcopt->plm_chement & PXMCHEMENT_PLM_RIGHT_MODIF)
	report_gs = 
	  g_string_append (report_gs, _("Polymer End Modif: RIGHT\n"));
	
      else
	report_gs = 
	  g_string_append (report_gs, _("Polymer End Modif: NONE\n\n"));
    }

  
  /* At this point, we can ask that the actual report be performed.
   */
  
  /* Tell for which sequence, the composition was performed.
   */
  g_string_append_printf (report_gs, 
			  _("Selected sequence: [%d-%d] --- "),
			  calcopt->start_idx + 1, 
			  calcopt->end_idx);

  if ((reportopt->oligomer_opt & PXM_OLM_REPORTOPT_SEQUENCE) ==
      PXM_OLM_REPORTOPT_SEQUENCE)
    {
      /* Remember that for reasons related to the dealing of the
	 graphical rendering of the polymer sequence in the editor,
	 the end_idx value that is returned by the
	 polyxmass_seqed_widget_get_selection_indices() function call
	 above is > the true value by one unit. So we have to reduce
	 the obtained value by one:
      */
      calcopt->end_idx--;
      report = pxmchem_polymer_make_codes_string (polymer,
						  calcopt->start_idx + 1,
						  calcopt->end_idx);
      g_assert (report != NULL);
      
      g_string_append_printf (report_gs, "%s\n", report);
      
      g_free (report);
    }
  

  widget = g_object_get_data (G_OBJECT (window), 
			      "isoelectric_point_entry");
  g_assert (widget != NULL);
  
  report = (gchar *) gtk_entry_get_text (GTK_ENTRY (widget));
  g_assert (report != NULL);
  
  g_string_append_printf (report_gs, _("Iso-electric Point: %s\n\n"), report);


  widget =  g_object_get_data (G_OBJECT (window), 
			       "hydrogen_potential_spinbutton");
  g_assert (widget != NULL);
  
  ph = gtk_spin_button_get_value (GTK_SPIN_BUTTON (widget));
  
  g_string_append_printf (report_gs, _("Hydrogen Potential: %.4lf\n\n"), ph);
  


  report_gs = g_string_append (report_gs,
			       _("Charges:\n"
			       "--------\n\n"));
  
  widget = g_object_get_data (G_OBJECT (window), 
			      "positive_charges_entry");
  g_assert (widget != NULL);
  
  report = (gchar *) gtk_entry_get_text (GTK_ENTRY (widget));
  g_assert (report != NULL);
  
  g_string_append_printf (report_gs, _("Positive: %s"), report);
  

  widget = g_object_get_data (G_OBJECT (window), 
			      "negative_charges_entry");
  g_assert (widget != NULL);
  
  report = (gchar *) gtk_entry_get_text (GTK_ENTRY (widget));
  g_assert (report != NULL);
  
  g_string_append_printf (report_gs, _("\tNegative: %s"), report);
  

  widget = g_object_get_data (G_OBJECT (window), 
			      "net_charge_entry");
  g_assert (widget != NULL);
  
  report = (gchar *) gtk_entry_get_text (GTK_ENTRY (widget));
  g_assert (report != NULL);
  
  g_string_append_printf (report_gs, _("\tNet: %s\n\n"), report);
  

  widget = g_object_get_data (G_OBJECT (window), 
			      "treated_pka_number_entry");
  g_assert (widget != NULL);

  report = (gchar *) gtk_entry_get_text (GTK_ENTRY (widget));
  g_assert (report != NULL);
  
  g_string_append_printf (report_gs, _("Number of pKa analyzed: %s\n\n\n"), 
			  report);

  report = report_gs->str;
  g_string_free (report_gs, FALSE);
  
  return report;
}



gchar *
polyxedit_pka_ph_pi_wnd_make_report_latex_format (PxmReportOpt* reportopt,
						  PxmWinMngmt* winmngmt)
{


  return NULL;
}







/* WINDOW LIFE-CYCLE FUNCTIONS.
 */
void
polyxedit_pka_ph_pi_wnd_really_close (GtkWidget *window)
{
  g_assert (window != NULL);
  

  /* This function is called as a GDestroyNotify callback function when
     the sequence editor window gets closed.
  */
  
  /* As a first step we want to un_register this window.
   */
  polyxmass_winmngmt_un_register_window (window, "POLYXEDIT");
  
  /* 
     Prior to closing the window, we want to make sure that no
     pending timed-out messages are there...
  */
  polyxmass_timeoutmsg_messages_remove ((GtkWindow *) window);
   
  gtk_widget_destroy (window);
}



gint
polyxedit_pka_ph_pi_wnd_delete_event  (GtkWidget *window, 
					GdkEventAny *event, 
					gpointer data)
{
  PxmEditCtxt *editctxt = data;
  
  gchar *help = NULL;
  

  g_assert (window != NULL);
  g_assert (editctxt != NULL);
  

  /* As a first step we want to un_register this window.
   */
  polyxmass_winmngmt_un_register_window (window, "POLYXEDIT");
  
  /* 
     Prior to closing the window, we want to make sure that no
     pending timed-out messages are there...
  */
  polyxmass_timeoutmsg_messages_remove ((GtkWindow *) window);
  
  /* This window pointer was set as a full datum to the sequence editor
     window, which means that we have to remove that pointer, without
     triggering the callback function call.
  */
  help = g_strdup_printf ("pka_ph_pi_wnd-%p", window);
  
  window = 
    g_object_steal_data (G_OBJECT (editctxt->
				   sequence_editor_wnd),
			 help);
  
  g_free (help);
  
  /* Let Gtk+ do the rest of the work.
   */
  return FALSE;
}


gint
polyxedit_pka_ph_pi_wnd_destroy_event  (GtkWidget *window, 
					 GdkEventAny *event, 
					 gpointer data)
{
  return FALSE;
}

